/*
 * Copyright (c) 2002, Jason Tishler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * A copy of the GNU General Public License can be found at
 * http://www.gnu.org/
 *
 * Written by Jason Tishler <jason@tishler.net>
 */

#include <iomanip>
#include "RebaseConfigBuilder.h"

RebaseConfigBuilder::RebaseConfigBuilder (const string &aFileName) :
  theStream (aFileName.c_str (), ios::binary)
{
  if (!theStream)
    {
      cerr << "Cannot open config file " + aFileName + " for writing" << endl;
      // FIXME: throw exception?
    }
}

RebaseConfigBuilder::~RebaseConfigBuilder ()
{
}

void
RebaseConfigBuilder::buildVersion ()
{
  theStream << theVersionHeader << endl;
}

void
RebaseConfigBuilder::buildVersionEntry (const string &aVersion)
{
  // Ignore version argument
  theStream << theVersion << endl;
}

void
RebaseConfigBuilder::buildFreeList ()
{
  theStream << endl;
  theStream << theFreeListHeader << endl;
}

void
RebaseConfigBuilder::buildFreeListEntry (unsigned long aBegin,
    					 unsigned long anEnd)
{
  theStream << hex << theHexPrefix << setw (8) << setfill ('0') << aBegin <<
    theFieldSep << theHexPrefix << setw (8) << setfill ('0') << anEnd << endl;
}

void
RebaseConfigBuilder::buildUsedList ()
{
  theStream << endl;
  theStream << theUsedListHeader << endl;
}

void
RebaseConfigBuilder::buildUsedListEntry (const string &aPathname,
					 unsigned long aBase,
					 unsigned long aSize)
{
  theStream << aPathname << theFieldSep << hex << theHexPrefix << setw (8) <<
    setfill ('0') << aBase << theFieldSep << theHexPrefix << setw (8) <<
    setfill ('0') << aSize << endl;
}

const string RebaseConfigBuilder::theVersionHeader = "[Version]";
const string RebaseConfigBuilder::theFreeListHeader = "[Free List]";
const string RebaseConfigBuilder::theUsedListHeader = "[Used List]";
const string RebaseConfigBuilder::theVersion = "1.0";
const string RebaseConfigBuilder::theHexPrefix = "0x";
const string RebaseConfigBuilder::theFieldSep = "\t";
