/*
 * Copyright (c) 2002, Jason Tishler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * A copy of the GNU General Public License can be found at
 * http://www.gnu.org/
 *
 * Written by Jason Tishler <jason@tishler.net>
 *
 */

#ifndef _RebaseConfigParser_h_
#define _RebaseConfigParser_h_

#include <fstream>
#include <vector>
#include <string>

class RebaseBuilder;

class RebaseConfigParser
{
public:
  RebaseConfigParser (const string &aFileName);
  ~RebaseConfigParser ();

  bool parse(RebaseBuilder &aBuilder);

private:
  typedef vector<string> Fields;

  static string getLine (istream &aStream);
  static bool isSpaceLine (const string &aLine);
  static void getFields (const string &aString, Fields &aFields);

  bool parseVersion (RebaseBuilder &aBuilder);
  bool parseFreeList (RebaseBuilder &aBuilder);
  bool parseUsedList (RebaseBuilder &aBuilder);

  string getHeader ();

  static const string theVersionHeader;
  static const string theFreeListHeader;
  static const string theUsedListHeader;

  ifstream theStream;
};

#endif // _RebaseConfigParser_h_
