/*
 * Copyright (c) 2002, Jason Tishler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * A copy of the GNU General Public License can be found at
 * http://www.gnu.org/
 *
 * Written by Jason Tishler <jason@tishler.net>
 */

#include <iostream>
#include <iomanip>
#include "RebaseFreeList.h"

RebaseFreeList::RebaseFreeList ()
{
}

RebaseFreeList::~RebaseFreeList ()
{
}

void
RebaseFreeList::insert (unsigned long aBegin, unsigned long anEnd)
{
  RebaseFreeListEntry anEntry (aBegin, anEnd);
  theFreeList.insert (anEntry);
}

// FIXME: aNewImageBase is *not* set!
bool
RebaseFreeList::remove (unsigned long anImageSize, unsigned long &aNewImageBase)
{
  bool status = false;

  for (FreeList::iterator i = theFreeList.begin (); i != theFreeList.end ();
       i++)
    {
      const RebaseFreeListEntry &anEntry = (*i);
      if (anImageSize <= anEntry.size)
	{
	  long aBegin = anEntry.begin;
	  long anEnd = anEntry.end - anImageSize;
	  theFreeList.erase (i);
	  if (anEnd > aBegin)
	    insert (aBegin, anEnd);
	  status = true;
	  break;
	}
    }

  return status;
}

void
RebaseFreeList::update (unsigned long aBegin, unsigned long anEnd)
{
  // FIXME: Is below kosher?
  FreeList::iterator below = theFreeList.end ();
  FreeList::iterator above = theFreeList.end ();

  for (FreeList::iterator i = theFreeList.begin (); i != theFreeList.end ();
       i++)
    {
      const RebaseFreeListEntry &anEntry = (*i);
      if (anEntry.end == aBegin)
	{
	  aBegin = anEntry.begin;
	  below = i;
	}
      else if (anEntry.begin == anEnd)
	{
	  anEnd = anEntry.end;
	  above = i;
	}
    }

  if (below != theFreeList.end ())
    theFreeList.erase (below);
  if (above != theFreeList.end ())
    theFreeList.erase (above);

  insert (aBegin, anEnd);
}


void
RebaseFreeList::print () const
{
  for (FreeList::iterator i = theFreeList.begin (); i != theFreeList.end ();
       i++)
    {
      const RebaseFreeListEntry &anEntry = (*i);
      cout << "size = " << hex << "0x" << setw (8) << setfill ('0') <<
	anEntry.size << ", begin = " << "0x" << setw (8) << setfill ('0') <<
	anEntry.begin << ", end = " << "0x" << setw (8) << setfill ('0') <<
	anEntry.end << endl;;
    }
}

RebaseFreeListEntry::RebaseFreeListEntry (unsigned long aBegin,
					  unsigned long anEnd) :
  size (anEnd - aBegin), begin (aBegin), end (anEnd)
{
}

bool
RebaseFreeListEntry::operator< (const RebaseFreeListEntry &rhs) const
{
  return size < rhs.size;
}
